fix_memory_leak_v3.patch
application/octet-stream
Filename: fix_memory_leak_v3.patch
Type: application/octet-stream
Part: 2
Patch
Format: unified
Series: patch v3
| File | + | − |
|---|---|---|
| src/backend/replication/pgoutput/pgoutput.c | 13 | 9 |
diff --git a/src/backend/replication/pgoutput/pgoutput.c b/src/backend/replication/pgoutput/pgoutput.c
index c9c952a278f..c715e8b8d4c 100644
--- a/src/backend/replication/pgoutput/pgoutput.c
+++ b/src/backend/replication/pgoutput/pgoutput.c
@@ -80,6 +80,7 @@ static void pgoutput_stream_prepare_txn(LogicalDecodingContext *ctx,
ReorderBufferTXN *txn, XLogRecPtr prepare_lsn);
static bool publications_valid;
+static MemoryContext pubctx = NULL;
static List *LoadPublications(List *pubnames);
static void publication_invalidation_cb(Datum arg, int cacheid,
@@ -430,6 +431,10 @@ pgoutput_startup(LogicalDecodingContext *ctx, OutputPluginOptions *opt,
"logical replication cache context",
ALLOCSET_DEFAULT_SIZES);
+ pubctx = AllocSetContextCreate(ctx->context,
+ "logical replication publication list context",
+ ALLOCSET_SMALL_SIZES);
+
ctx->output_plugin_private = data;
/* This plugin uses binary protocol. */
@@ -1696,9 +1701,9 @@ pgoutput_origin_filter(LogicalDecodingContext *ctx,
/*
* Shutdown the output plugin.
*
* Note, we don't need to clean the data->context and data->cachectx as
* they are child contexts of the ctx->context so they will be cleaned up by
* logical decoding machinery.
*/
static void
pgoutput_shutdown(LogicalDecodingContext *ctx)
@@ -2024,12 +2029,11 @@ get_rel_sync_entry(PGOutputData *data, Relation relation)
/* Reload publications if needed before use. */
if (!publications_valid)
{
- oldctx = MemoryContextSwitchTo(CacheMemoryContext);
- if (data->publications)
- {
- list_free_deep(data->publications);
- data->publications = NIL;
- }
+ Assert(pubctx);
+
+ MemoryContextReset(pubctx);
+ oldctx = MemoryContextSwitchTo(pubctx);
+
data->publications = LoadPublications(data->publication_names);
MemoryContextSwitchTo(oldctx);
publications_valid = true;
@@ -1696,9 +1701,9 @@ pgoutput_origin_filter(LogicalDecodingContext *ctx,
* Shutdown the output plugin.
*
- * Note, we don't need to clean the data->context and data->cachectx as
- * they are child contexts of the ctx->context so they will be cleaned up by
- * logical decoding machinery.
+ * Note, we don't need to clean the data->context, data->cachectx and pubctx
+ * as they are child contexts of the ctx->context so they will be cleaned up
+ * by logical decoding machinery.
*/
static void